<?php

namespace Picea\Extension;

class UrlExtension implements Extension {

    protected string $urlBase;
    
    protected string $assetToken;
    
    public array $tokens = [ "url" , "route", "asset" ];
    
    public function __construct(string $urlBase = "", string $assetToken = "") {
        $this->urlBase = $urlBase;
        $this->assetToken = $assetToken;
    }

    public function parse(/*\Picae\Compiler\Context*/ &$context, ?string $arguments, string $token) : ?string 
    {
        switch($token) {
            case "asset": 
                return "<?php echo \$___picea->compiler->getExtensionFromToken('$token')->buildAssetUrl($arguments) ?>";
                
            case "route": 
                return "<?php echo \$___picea->compiler->getExtensionFromToken('$token')->buildUrl($arguments) ?>";
                
            case "url": 
                return "<?php echo \$___picea->compiler->getExtensionFromToken('$token')->buildUrl($arguments) ?>";
        }
        
        return null;
    }
    
    public function buildUrl(string $uri, array $parameters = []) : string
    {
        return $this->url() . $uri . ( $parameters ? "?" . http_build_query($parameters) : "" );
    }
    
    public function buildAssetUrl(string $uri, array $parameters = []) : string
    {
        return $this->url() . $uri . "?" . http_build_query( array_replace([ 'token' => $this->assetToken ], $parameters) );
    }
    
    public function url() : string
    {
        return $this->scheme() . $this->domain() . $this->base();
    }
    
    /**
     * Return URI formatted
     */
    protected function uri(bool $queryString = false) : string
    {
        $uri = ( $_SERVER['REQUEST_URI'] ?? $_SERVER['PHP_SELF'] ?? "" ) . ( $queryString ? $this->queryString() : "" );

        if ( ! $queryString ) {
            $uri = explode('?', $uri, 2)[0];
        }

        if ( ($base = $this->config['base'] ?? false) && ( stripos($uri, $base) === 0 ) ) {
            $uri = substr($uri, strlen($base));
        }
        
        return '/' . ltrim($uri, '/');
    }
    
    /**
     * Return query string
     */
    protected function queryString() : string 
    {
        return isset($_SERVER['QUERY_STRING']) ? '?' . $_SERVER['QUERY_STRING'] : "";
    }
    
    protected function scheme() : string 
    {
        return ( $this->isHttps() ? "https" : "http" ) . "://";
    }
    
    protected function base() : string 
    {
        return $this->urlBase ? $this->urlBase . "/" : "";
    }
    
    /**
     * Check if Uri's segments are valid
     * @param array $segments The uri's segments
     */
    protected function secure($segments = []) : array 
    {
        return array_diff($segments, [ '..', '://' ]);
    }

    protected function domain() : string
    {
        return strtolower($_SERVER['HTTP_HOST']);
    }
    
    protected function isDefaultPort() : bool 
    {
        return in_array($_SERVER['SERVER_PORT'], [ 80, 443 ]);
    }
    
    protected function isHttps() : bool 
    {
        $header = in_array('https', array_map("strtolower", [ 
            $_SERVER['X-Url-Scheme'] ?? "",
            $_SERVER['Front-End-Https'] ?? "", 
            $_SERVER['X-Forwarded-Ssl'] ?? "",
            $_SERVER['X-Forwarded-Proto'] ?? "", 
            $_SERVER['X-Forwarded-Protocol'] ?? "",
        ]));
        
        return $header 
            || ( "443" === $_SERVER['SERVER_PORT'] ?? "" ) 
            || ( "https" === $_SERVER['REQUEST_SCHEME'] ?? "http" ) 
            || ( "off" !== strtolower($_SERVER['HTTPS'] ?? "off") );
    }
}